Write a Python script to train a machine learning model that solves the TASK outlined below, 
using the approach outlined in the plan below.

# TASK:
{{ problem_statement }}

# PLAN:
{{ plan }}

# PREVIOUS ATTEMPTS, IF ANY:
{{history}}

# INSTRUCTIONS
Only return the code of the transformation script, no explanations outside the code. Any explanation should
be in the comments in the code itself, but your overall answer must only consist of the code script.

The script MUST be wrapped in a function called `transformation` and follow this format:

```python

import click


@click.command()
@click.option('--input', help='Input parquet file')
@click.option('--output', help='Output parquet file.')
def transformation(input, output):
    # The transformation code should be here 

if __name__ == "__main__":
    transformation()

``` 

- The code validation will be like this: 
```bash
python transformation.py --input <input> --output <output>
```

- Use only {{ allowed_packages }}. Do NOT use any packages that are not part of this list of the Python standard library.
