Review the solution to fix issues.

# TASK:
{{problem_statement}}

# PLAN:
{{plan}}

# CODE SOLUTION:
```python
{{transformation_code}}
```

# ERRORS:
{{problems}}


Your task is to suggest a single, actionable improvement to the code solution that will fix the issues. Do NOT
return the full code solution, but return a clear explanation of what needs to change, and a code snippet showing
the change. Note that only the following packages are allowed: {{ allowed_packages }}.