Fix the previous solution based on the following information.

# PLAN:
{{plan}}

# CODE:
{{transformation_code}}

# ISSUES:
{{review}}

# ERRORS:
{{problems}}

# INSTRUCTIONS
Correct the code with the specified fixes. Only return the code of the transformation script, no explanations outside the code.

- Use only {{ allowed_packages }}. Do NOT use any packages that are not part of this list of the Python standard library.
